@@ -21,6 +21,11 @@
 
 endpoints <-
 function(x,on='months',k=1) {
+
+  if(k < 1) {
+    stop("'k' must be > 0")
+  }
+
   if(timeBased(x)) {
     NR <- length(x)
     x <- xts(, order.by=x)
@@ -39,22 +44,37 @@ function(x,on='months',k=1) {
   # posixltindex is costly in memory (9x length of time)
   # make sure we really need it
   if(on %in% c('years','quarters','months','weeks','days'))
-    posixltindex <- as.POSIXlt(.POSIXct(.index(x)),tz=indexTZ(x))
+    posixltindex <- as.POSIXlt(.POSIXct(.index(x)),tz=tzone(x))
+
+  include_last <- function(x, k) {
+    len <- length(x)
+    i <- seq(1L ,len, k)
+    if(i[length(i)] != len) {
+      i <- c(i, len)
+    }
+    ep[i]
+  }
 
   switch(on,
     "years" = {
       as.integer(c(0, which(diff(posixltindex$year %/% k + 1) != 0), NR))
     },
     "quarters" = {
-      ixqtr <- posixltindex$year * 100L + 190000L + posixltindex$mon %/% 3 + 1
-      as.integer(c(0,which(diff(ixqtr) != 0),NR))
+      ixyear <- posixltindex$year * 100L + 190000L
+      ixqtr <- ixyear + posixltindex$mon %/% 3L + 1L
+      ep <- c(0L, which(diff(ixqtr) != 0L), NR)
+      if(k > 1) {
+        ep <- include_last(ep, k)
+      }
+      ep
     },
     "months" = {
       ixmon <- posixltindex$year * 100L + 190000L + posixltindex$mon
       ep <- .Call("endpoints", ixmon, 1L, 1L, addlast, PACKAGE='xts')
-      if(k > 1)
-        ep[seq(1,length(ep),k)]
-      else ep
+      if(k > 1) {
+        ep <- include_last(ep, k)
+      }
+      ep
     },
     "weeks" = {
       .Call("endpoints", .index(x)+3L*86400L, 604800L, k, addlast, PACKAGE='xts')
@@ -75,11 +95,13 @@ function(x,on='months',k=1) {
     },
     "ms" = ,
     "milliseconds" = {
-      .Call("endpoints", .index(x)%/%.001, 1L, k, addlast, PACKAGE='xts')
+      sec2ms <- .index(x) * 1e3
+      .Call("endpoints", sec2ms, 1L, k, addlast, PACKAGE='xts')
     },
     "us" = ,
     "microseconds" = {
-      .Call("endpoints", .index(x)%/%.000001, 1L, k, addlast, PACKAGE='xts')
+      sec2us <- .index(x) * 1e6
+      .Call("endpoints", sec2us, 1L, k, addlast, PACKAGE='xts')
     }
   )
 }
@@ -122,9 +144,10 @@ function (year = 1970,
     }
     # strptime has an issue (bug?) which returns NA when passed
     # 1969-12-31-23-59-59; pass 58.9 secs instead.
+    sysTZ <- Sys.getenv("TZ")
     if (length(c(year, month, day, hour, min, sec)) == 6 &&
         all(c(year, month, day, hour, min, sec) == c(1969, 12, 31, 23, 59, 59)) &&
-        Sys.getenv("TZ") %in% c("", "GMT", "UTC"))
+        (sysTZ == "" || isUTC(sysTZ)))
         sec <- sec-1
     ISOdatetime(year, month, day, hour, min, sec, tz)
 }
