@@ -19,6 +19,80 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+.subsetTimeOfDay <- function(x, fromTimeString, toTimeString) {
+  validateTimestring <- function(time) {
+    h    <- "(?:[01]?\\d|2[0-3])"
+    hm   <- paste0(h, "(?::?[0-5]\\d)")
+    hms  <- paste0(hm, "(?::?[0-5]\\d)")
+    hmsS <- paste0(hms, "(?:\\.\\d{1,9})?")
+    pattern <- paste(h, hm, hms, hmsS, sep = ")$|^(")
+    pattern <- paste0("^(", pattern, "$)")
+
+    if (!grepl(pattern, time)) {
+      # FIXME: this isn't necessarily true...
+      # colons aren't required, and neither are all of the components
+      stop("Supply time-of-day subsetting in the format of T%H:%M:%OS/T%H:%M:%OS",
+           call. = FALSE)
+    }
+  }
+
+  validateTimestring(fromTimeString)
+  validateTimestring(toTimeString)
+
+  getTimeComponents <- function(time) {
+    # split on decimal point
+    time. <- strsplit(time, ".", fixed = TRUE)[[1]]
+    hms <- time.[1L]
+
+    # ensure hms string has even nchar
+    nocolon <- gsub(":", "", hms, fixed = TRUE)
+    if (nchar(nocolon) %% 2 > 0) {
+      # odd nchar means leading zero is omitted from hours
+      # all other components require zero padding
+      hms <- paste0("0", hms)
+    }
+    # add colons
+    hms <- gsub("(.{2}):?", ":\\1", hms, perl = TRUE)
+    # remove first character (a colon)
+    hms <- substr(hms, 2, nchar(hms))
+
+    # extract components
+    comp <- strsplit(hms, ":", fixed = TRUE)[[1]]
+    complist <-
+      list(hour = comp[1L],
+           min = comp[2L],
+           sec = comp[3L],
+           subsec = time.[2L])
+    # remove all missing components
+    complist <- complist[!vapply(complist, is.na, logical(1))]
+    # convert to numeric
+    complist <- lapply(complist, as.numeric)
+
+    # add timezone and return
+    c(tz = "UTC", complist)
+  }
+
+  # first second in period (no subseconds)
+  from <- do.call(firstof, getTimeComponents(fromTimeString)[-5L])
+  secBegin <- as.numeric(from) %% 86400L
+
+  # last second in period
+  to <- do.call(lastof, getTimeComponents(toTimeString))
+  secEnd <- as.numeric(to) %% 86400L
+
+  # do subsetting
+  tz <- tzone(x)
+  secOfDay <- as.POSIXlt(index(x), tz = tz)
+  secOfDay <- secOfDay$hour * 60 * 60 + secOfDay$min * 60 + secOfDay$sec
+
+  if (secBegin <= secEnd) {
+    i <- secOfDay >= secBegin & secOfDay <= secEnd
+  } else {
+    i <- secOfDay >= secBegin | secOfDay <= secEnd
+  }
+  which(i)
+}
+
 .subset_xts <- function(x, i, j, ...) {
   if(missing(i)) {
     i <- 1:NROW(x)
@@ -33,15 +107,17 @@
 function(x, i, j, drop = FALSE, which.i=FALSE,...) 
 {
     USE_EXTRACT <- FALSE # initialize to FALSE
-    if(is.null(dim(x))) {
+
+    dimx <- dim(x)
+    if(is.null(dimx)) {
       nr <- length(x)
       if(nr==0 && !which.i)
         return( xts(rep(NA,length(index(x))), index(x))[i] )
       nr <- length(.index(x))
       nc <- 1L
     } else {
-      nr <- nrow(x)
-      nc <- ncol(x)
+      nr <- dimx[1L]
+      nc <- dimx[2L]
     }
     
     if(!missing(i)) {
@@ -68,27 +144,27 @@ function(x, i, j, drop = FALSE, which.i=FALSE,...)
       i <- which(i) #(1:NROW(x))[rep(i,length.out=NROW(x))]
     } else
     if (is.character(i)) {
-      # is i of the format T/T?
       if(length(i) == 1 && !identical(integer(),grep("^T.*?/T",i[1]))) {
-      #if(grepl("^T.*?/T",i[1]) && length(i) == 1) {
-        i <- gsub("T|:","",i)
-        i <- strsplit(i, "/")[[1]]
-        i <- .makeISO8601TT(x, i[1],i[2])
-      }
-      # enables subsetting by date style strings
-      # must be able to process - and then allow for operations???
+        # is i of the format T/T?
+        ii <- gsub("T", "", i, fixed = TRUE)
+        ii <- strsplit(ii, "/", fixed = TRUE)[[1L]]
+        i <- .subsetTimeOfDay(x, ii[1L], ii[2L])
+      } else {
+        # enables subsetting by date style strings
+        # must be able to process - and then allow for operations???
 
-      i.tmp <- NULL
-      tz <- as.character(indexTZ(x)) # ideally this moves to attr(index,"tzone")
-      i_len <- length(i)
+        i.tmp <- NULL
+        tz <- as.character(tzone(x))
 
-      for(ii in i) {
-        adjusted.times <- .parseISO8601(ii, .index(x)[1], .index(x)[nr], tz=tz)
-        if(length(adjusted.times) > 1) {
-          i.tmp <- c(i.tmp, index_bsearch(.index(x), adjusted.times$first.time, adjusted.times$last.time))
+        for(ii in i) {
+          adjusted.times <- .parseISO8601(ii, .index(x)[1], .index(x)[nr], tz=tz)
+          if(length(adjusted.times) > 1) {
+            i.tmp <- c(i.tmp, index_bsearch(.index(x), adjusted.times$first.time, adjusted.times$last.time))
+          }
         }
+        i <- i.tmp
       }
-      i <- i.tmp
+      i_len <- length(i)
 
       if(i_len == 1L)  # IFF we are using ISO8601 subsetting
         USE_EXTRACT <- TRUE
@@ -117,7 +193,10 @@ function(x, i, j, drop = FALSE, which.i=FALSE,...)
         i <- seq_len(nr)
 
       if(length(x)==0) {
-        x.tmp <- .xts(rep(NA,length(i)), .index(x)[i], dimnames=list(NULL, colnames(x)))
+        cdata <- rep(NA, length(i))
+        storage.mode(cdata) <- storage.mode(x)
+        x.tmp <- .xts(cdata, .index(x)[i], tclass(x), tzone(x),
+                      dimnames = list(NULL, colnames(x)))
         return(x.tmp)
       } else {
         if(USE_EXTRACT) {
@@ -164,8 +243,11 @@ function(x, i, j, drop = FALSE, which.i=FALSE,...)
     if(length(j) == 0 || (length(j)==1 && (is.na(j) || j==0))) {
       if(missing(i))
         i <- seq_len(nr)
-      return(.xts(coredata(x)[i,j,drop=FALSE], index=.index(x)[i],
-                  .indexCLASS=indexClass(x), .indexTZ=indexTZ(x)))
+
+      output <- .xts(coredata(x)[i,j,drop=FALSE], .index(x)[i],
+                     tclass(x), tzone(x), class = class(x))
+      xtsAttributes(output) <- xtsAttributes(x)
+      return(output)
     } 
     if(missing(i))
       return(.Call("extract_col", x, as.integer(j), drop, 1, nr, PACKAGE='xts'))
@@ -251,7 +333,8 @@ window_idx <- function(x, index. = NULL, start = NULL, end = NULL)
     usr_idx <- TRUE
     idx <- .index(x)
 
-    index. <- .toPOSIXct(index., indexTZ(x))
+    index. <- .toPOSIXct(index., tzone(x))
+    index. <- unclass(index.)
     index. <- index.[!is.na(index.)]
     if(is.unsorted(index.)) {
       # index. must be sorted for index_bsearch
@@ -265,15 +348,16 @@ window_idx <- function(x, index. = NULL, start = NULL, end = NULL)
     match <- idx[base_idx] == index.
     base_idx <- base_idx[match]
     index. <- index.[match]
+    index. <- .POSIXct(index., tz = tzone(x))
     if(length(base_idx) < 1) return(x[NULL,])
   }
 
   if(!is.null(start)) {
-    start <- .toPOSIXct(start, indexTZ(x))
+    start <- .toPOSIXct(start, tzone(x))
   }
 
   if(!is.null(end)) {
-    end <- .toPOSIXct(end, indexTZ(x))
+    end <- .toPOSIXct(end, tzone(x))
   }
 
   firstlast <- index_bsearch(index., start, end)
@@ -297,6 +381,11 @@ window.xts <- function(x, index. = NULL, start = NULL, end = NULL, ...)
 {
   if(is.null(start) && is.null(end) && is.null(index.)) return(x)
 
+  # dispatch to window.zoo() for yearmon and yearqtr
+  if(any(tclass(x) %in% c("yearmon", "yearqtr"))) {
+    return(NextMethod(.Generic))
+  }
+
   firstlast <- window_idx(x, index., start, end) # firstlast may be NULL
 
   .Call('_do_subset_xts',
