@@ -25,6 +25,48 @@
 #include <Rdefines.h>
 #include "xts.h"
 
+SEXP xts_make_names(SEXP colnames, SEXP env)
+{
+  SEXP s, t, unique;
+  PROTECT(s = t = allocList(3));
+  SET_TYPEOF(s, LANGSXP);
+  SETCAR(t, install("make.names"));
+  t = CDR(t);
+  SETCAR(t, colnames);
+  t = CDR(t);
+  PROTECT(unique = allocVector(LGLSXP, 1));
+  LOGICAL(unique)[0] = 1;
+  SETCAR(t, unique);
+  SET_TAG(t, install("unique"));
+  SEXP res = PROTECT(eval(s, env));
+  UNPROTECT(3);
+  return(res);
+}
+
+SEXP xts_colname_suffixes(SEXP colnames, SEXP suffixes, SEXP env)
+{
+  SEXP s, t;
+  PROTECT(s = t = allocList(4));
+  SET_TYPEOF(s, LANGSXP);
+
+  SETCAR(t, install("paste"));
+  t = CDR(t);
+
+  SETCAR(t, colnames);
+  t = CDR(t);
+
+  SETCAR(t, suffixes);
+  t = CDR(t);
+
+  SETCAR(t, mkString(""));
+  SET_TAG(t, install("sep"));
+
+  SEXP res = PROTECT(eval(s, env));
+
+  UNPROTECT(2);
+  return(res);
+}
+
 /* 
 
   This is a merge_join algorithm used to
@@ -47,8 +89,9 @@ SEXP do_merge_xts (SEXP x, SEXP y,
                    SEXP colnames, 
                    SEXP suffixes,
                    SEXP retside,
+                   SEXP check_names,
                    SEXP env,
-                   int coerce)
+                   SEXP coerce)
 {
   int nrx, ncx, nry, ncy, len;
   int left_join, right_join;
@@ -57,7 +100,7 @@ SEXP do_merge_xts (SEXP x, SEXP y,
   int ij_original, ij_result;
   int p = 0;
   SEXP xindex, yindex, index, result, attr, len_xindex;
-  SEXP s, t, unique;
+  SEXP s, t;
 
   int *int_result=NULL, *int_x=NULL, *int_y=NULL, int_fill=0;
   int *int_index=NULL, *int_xindex=NULL, *int_yindex=NULL;
@@ -83,7 +126,7 @@ SEXP do_merge_xts (SEXP x, SEXP y,
   PROTECT( xindex = getAttrib(x, xts_IndexSymbol) ); p++;
 
   /* convert to xts object if needed */
-  if( !isXts(y) ) {
+  if( !Rf_asInteger(isXts(y)) ) {
     PROTECT(s = t = allocList(4)); p++;
     SET_TYPEOF(s, LANGSXP);
     SETCAR(t, install("try.xts")); t = CDR(t);
@@ -99,7 +142,7 @@ SEXP do_merge_xts (SEXP x, SEXP y,
 
   mode = TYPEOF(x);
 
-  if( isXts(y) ) {
+  if( Rf_asInteger(isXts(y)) ) {
     PROTECT( yindex = getAttrib(y, xts_IndexSymbol) ); p++;
   } else {
     PROTECT( yindex = getAttrib(x, xts_IndexSymbol) ); p++;
@@ -127,6 +170,16 @@ SEXP do_merge_xts (SEXP x, SEXP y,
   }
 
   len = nrx + nry;
+  if (len < 1 && ncx < 1 && ncy < 1) {
+    /* nothing to do, return empty xts object */
+    SEXP s, t;
+    PROTECT(s = t = allocList(1)); p++;
+    SET_TYPEOF(s, LANGSXP);
+    SETCAR(t, install("xts"));
+    SEXP out = PROTECT(eval(s, env)); p++;
+    UNPROTECT(p);
+    return out;
+  }
 
   /* at present we are failing the call if the indexing is of
      mixed type.  This should probably instead simply coerce
@@ -250,7 +303,7 @@ SEXP do_merge_xts (SEXP x, SEXP y,
      either here or in the calling R code.  I suspect here is
      more useful if other function can call the C code as well. 
      If objects are not the same type, convert to REALSXP. */
-  if( coerce || TYPEOF(x) != TYPEOF(y) ) {
+  if( Rf_asInteger(coerce) || TYPEOF(x) != TYPEOF(y) ) {
     PROTECT( x = coerceVector(x, REALSXP) ); p++;
     PROTECT( y = coerceVector(y, REALSXP) ); p++;
   }
@@ -320,8 +373,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ];
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ];
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ];
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0));
@@ -408,8 +461,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ];
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ];
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ];
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0));
@@ -523,8 +576,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ];
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ];
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ];
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0));
@@ -556,8 +609,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ];
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ];
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ];
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0));
@@ -618,8 +671,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ];
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ];
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ];
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0));
@@ -708,8 +761,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ]; //NA_REAL;
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ]; //NA_REAL;
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ]; //NA_REAL;
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0)); //NA_STRING);
@@ -839,8 +892,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ];
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ];
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ];
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0));
@@ -866,6 +919,7 @@ SEXP do_merge_xts (SEXP x, SEXP y,
           switch( mode ) {
             case LGLSXP:
               LOGICAL(result)[ ij_result ] = LOGICAL(fill)[ 0 ]; //NA_INTEGER;
+              break;
             case INTSXP:
               int_result[ ij_result ] = int_fill;
               break;
@@ -873,8 +927,8 @@ SEXP do_merge_xts (SEXP x, SEXP y,
               REAL(result)[ ij_result ] = REAL(fill)[ 0 ]; //NA_REAL;
               break;
             case CPLXSXP:
-              COMPLEX(result)[ ij_result ].r = REAL(fill)[ 0 ]; //NA_REAL;
-              COMPLEX(result)[ ij_result ].i = REAL(fill)[ 0 ]; //NA_REAL;
+              COMPLEX(result)[ ij_result ].r = COMPLEX(fill)[ 0 ].r;
+              COMPLEX(result)[ ij_result ].i = COMPLEX(fill)[ 0 ].i;
               break;
             case STRSXP:
               SET_STRING_ELT(result, ij_result, STRING_ELT(fill, 0)); //NA_STRING);
@@ -963,15 +1017,18 @@ SEXP do_merge_xts (SEXP x, SEXP y,
           }
         }
       }
-      SET_VECTOR_ELT(dimnames, 0, R_NilValue);  // ROWNAMES are NULL
 
-      PROTECT(s = t = allocList(3)); p++;
-      SET_TYPEOF(s, LANGSXP);
-      SETCAR(t, install("make.names")); t = CDR(t);
-      SETCAR(t, newcolnames); t = CDR(t);
-      PROTECT(unique = allocVector(LGLSXP, 1)); p++;  LOGICAL(unique)[0] = 1;
-      SETCAR(t, unique);  SET_TAG(t, install("unique"));
-      SET_VECTOR_ELT(dimnames, 1, eval(s, env));
+      // add suffixes
+      if(R_NilValue != suffixes) {
+        newcolnames = PROTECT(xts_colname_suffixes(newcolnames, suffixes, env)); p++;
+      }
+
+      SET_VECTOR_ELT(dimnames, 0, R_NilValue);  // ROWNAMES are NULL
+      if(LOGICAL(check_names)[0]) {
+        SET_VECTOR_ELT(dimnames, 1, xts_make_names(newcolnames, env));
+      } else {
+        SET_VECTOR_ELT(dimnames, 1, newcolnames);
+      }
  
       //SET_VECTOR_ELT(dimnames, 1, newcolnames); // COLNAMES are passed in
       setAttrib(result, R_DimNamesSymbol, dimnames);
@@ -984,9 +1041,6 @@ SEXP do_merge_xts (SEXP x, SEXP y,
   setAttrib(result, xts_IndexSymbol, index);
   if(LOGICAL(retclass)[0])
     setAttrib(result, R_ClassSymbol, getAttrib(x, R_ClassSymbol));
-  setAttrib(result, xts_IndexClassSymbol, getAttrib(x, xts_IndexClassSymbol));
-  setAttrib(result, xts_IndexTZSymbol, getAttrib(x, xts_IndexTZSymbol));
-  setAttrib(result, xts_IndexFormatSymbol, getAttrib(x, xts_IndexFormatSymbol));
   setAttrib(result, xts_ClassSymbol, getAttrib(x, xts_ClassSymbol));
   copy_xtsAttributes(x, result);
 
@@ -1001,7 +1055,7 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   SEXP _x, _y, xtmp, result, _INDEX;
   /* colnames should be renamed as suffixes, as colnames need to be added at the C level */
   SEXP all, fill, retc, retclass, symnames,
-       suffixes, rets, retside, env, tzone;
+       suffixes, rets, retside, env, tzone, check_names;
   int nr, nc, ncs=0;
   int index_len;
   int i, n=0, P=0;
@@ -1016,20 +1070,22 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   args = CDR(args); retside = CAR(args);
   args = CDR(args); env = CAR(args);
   args = CDR(args); tzone = CAR(args);
+  args = CDR(args); check_names = CAR(args);
   args = CDR(args);
   // args should now correspond to the ... objects we are looking to merge 
   argstart = args; // use this to rewind list...
 
   n = 0;
   int type_of;
-  int coerce_to_double=0;
+  SEXP coerce = PROTECT(ScalarInteger(0)); P++;
+
   if(args != R_NilValue) type_of = TYPEOF(CAR(args));
   while(args != R_NilValue) {
     if(length(CAR(args)) > 0) {
       ncs += ncols(CAR(args));
       /* need to convert all objects if one non-zero-width needs to be converted */
       if(TYPEOF(CAR(args)) != type_of) {
-        coerce_to_double = 1;
+        INTEGER(coerce)[0] = 1;
       }
     }
     args = CDR(args);
@@ -1054,7 +1110,7 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   args = CDR(args);
 
   int leading_non_xts = 0;
-  while( !isXts(_x) ) {
+  while( !Rf_asInteger(isXts(_x)) ) {
     if( args == R_NilValue ) error("no xts object to merge");
     leading_non_xts = 1;
     /*warning("leading non-xts objects may have been dropped");*/
@@ -1064,6 +1120,7 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   /* test for NULLs that may be present from cbind dispatch */
   if(!leading_non_xts) { /* leading non-xts in 2 case scenario was igoring non-xts value */
     if(n < 3 && (args == R_NilValue || (isNull(CAR(args)) && length(args) == 1))) {/* no y arg or y==NULL */
+      UNPROTECT(P);
       return(_x);
     }
   }
@@ -1100,8 +1157,9 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
                                              R_NilValue,
                                              R_NilValue,
                                              rets,
+                                             check_names,
                                              env,
-                                             coerce_to_double), &idx); P++;
+                                             coerce), &idx); P++;
 
     /* merge all objects into one zero-width common index */
     while(args != R_NilValue) { 
@@ -1114,8 +1172,9 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
                                         R_NilValue,
                                         R_NilValue,
                                         rets,
+                                        check_names,
                                         env,
-                                        coerce_to_double), idx);
+                                        coerce), idx);
       }
       args = CDR(args);
     }
@@ -1124,21 +1183,8 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   
     args = argstart; // reset args
     int ii, jj, iijj, jj_result;
-    int *int_result=NULL, *int_xtmp=NULL;
-    double *real_result=NULL, *real_xtmp=NULL;
 
     PROTECT(result = allocVector(TYPEOF(_INDEX), index_len * ncs)); P++;
-    switch(TYPEOF(result)) {
-      case LGLSXP:
-      case INTSXP:
-        int_result = INTEGER(result);
-        break;
-      case REALSXP:
-        real_result = REAL(result);
-        break;
-      default:
-        error("unsupported data type");
-    }
 
     SEXP ColNames, NewColNames;
     PROTECT(NewColNames = allocVector(STRSXP, ncs)); P++;
@@ -1164,46 +1210,98 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
                         /*colnames*/R_NilValue,
                                     R_NilValue,
                                     retside,
+                                    check_names,
                                     env,
-                                    coerce_to_double), idxtmp);
+                                    coerce), idxtmp);
 
       nr = nrows(xtmp);
       nc = (0 == nr) ? 0 : ncols(xtmp);  // ncols(numeric(0)) == 1
       ncs += nc;
+
+      /* Use colnames from merged object, if it has them. Otherwise, use
+       * use deparsed names */
       REPROTECT(ColNames = getAttrib(CAR(args),R_DimNamesSymbol), cnmtmp);
+      SEXP colnames = R_NilValue;
+      if(R_NilValue != ColNames) {
+        colnames = VECTOR_ELT(ColNames, 1);
+      }
+      if(R_NilValue == colnames) {
+        for(jj=0; jj < nc; jj++) {
+          SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(symnames,i+jj));
+        }
+      } else {
+        for(jj=0; jj < nc; jj++) {
+          SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(colnames,  jj));
+        }
+      }
+
       switch(TYPEOF(xtmp)) { // by type, insert merged data into result object
         case LGLSXP:
-        case INTSXP:
-          int_xtmp = INTEGER(xtmp);
-          for(jj=0; jj < nc; jj++) {
-            if(!isNull(ColNames) && !isNull(VECTOR_ELT(ColNames,1))) {
-              /* if merged object has colnames, use these, otherwise use deparse names */
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(VECTOR_ELT(ColNames,1),jj));
-            } else {
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(symnames,i+jj));
+          {
+            int *xtmp_ = LOGICAL(xtmp);
+            int *result_ = LOGICAL(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
             }
-            for(ii=0; ii < nr; ii++) {
-              iijj = ii + jj * nr;
-              jj_result = ii + ( (i+jj) * nr);
-              int_result[ jj_result ] = int_xtmp[ iijj ];
+          }
+          break;
+        case INTSXP:
+          {
+            int *xtmp_ = INTEGER(xtmp);
+            int *result_ = INTEGER(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
             }
           }
           break;
         case REALSXP:
-          real_xtmp = REAL(xtmp);
-          for(jj=0; jj < nc; jj++) {
-            if(!isNull(ColNames) && !isNull(VECTOR_ELT(ColNames,1))) {
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(VECTOR_ELT(ColNames,1),jj));
-            } else {
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(symnames,i+jj));
-            }
-            for(ii=0; ii < nr; ii++) {
-              iijj = ii + jj * nr;
-              jj_result = ii + ( (i+jj) * nr);
-              real_result[ jj_result ] = real_xtmp[ iijj ];
+          {
+            double *xtmp_ = REAL(xtmp);
+            double *result_ = REAL(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
             }
           }
           break;
+        case CPLXSXP:
+          {
+            Rcomplex *xtmp_ = COMPLEX(xtmp);
+            Rcomplex *result_ = COMPLEX(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
+            }
+          }
+          break;
+        case STRSXP:
+          {
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                SET_STRING_ELT(result, jj_result, STRING_ELT(xtmp, iijj));
+              }
+            }
+          }
+          break;
+        default:
+          error("unsupported data type");
+          break;
       }
     }
 
@@ -1218,20 +1316,21 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
       PROTECT(dimnames = allocVector(VECSXP, 2)); P++;
       SET_VECTOR_ELT(dimnames, 0, R_NilValue); // rownames are always NULL in xts
 
+      // Add suffixes
+      if(R_NilValue != suffixes) {
+        NewColNames = PROTECT(xts_colname_suffixes(NewColNames, suffixes, env)); P++;
+      }
+
       /* colnames, assure they are unique before returning */
-      SEXP s, t, unique;
-      PROTECT(s = t = allocList(3)); P++;
-      SET_TYPEOF(s, LANGSXP);
-      SETCAR(t, install("make.names")); t = CDR(t);
-      SETCAR(t, NewColNames); t = CDR(t);
-      PROTECT(unique = allocVector(LGLSXP, 1)); P++;  LOGICAL(unique)[0] = 1;
-      SETCAR(t, unique);  SET_TAG(t, install("unique"));
-      SET_VECTOR_ELT(dimnames, 1, eval(s, env));
+      if(LOGICAL(check_names)[0]) {
+        SET_VECTOR_ELT(dimnames, 1, xts_make_names(NewColNames, env));
+      } else {
+        SET_VECTOR_ELT(dimnames, 1, NewColNames);
+      }
       setAttrib(result, R_DimNamesSymbol, dimnames);
     }
 
     SET_xtsIndex(result, GET_xtsIndex(_INDEX));
-    SET_xtsIndexTZ(result, GET_xtsIndexTZ(_INDEX));
     copy_xtsCoreAttributes(_INDEX, result);
     copy_xtsAttributes(_INDEX, result);
 
@@ -1245,8 +1344,9 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
                             symnames /*R_NilValue*/,
                             suffixes,
                              retside,
+                         check_names,
                                  env,
-                    coerce_to_double)); P++;
+                              coerce)); P++;
   }
 
   SEXP index_tmp = getAttrib(result, xts_IndexSymbol);
@@ -1259,7 +1359,6 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   }
   copyMostAttrib(getAttrib(_x,xts_IndexSymbol), index_tmp);
   setAttrib(result, xts_IndexSymbol, index_tmp);
-  setAttrib(result, xts_IndexTZSymbol, getAttrib(index_tmp, xts_IndexTzoneSymbol));
 
   UNPROTECT(P);
   return(result);
